@@ -1,5 +1,5 @@
 /*
-  $Id: document.h 13851 2012-01-03 08:48:59Z sloot $
+  $Id: document.h 14474 2012-03-19 13:04:14Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/libfolia/trunk/include/folia/document.h $
 
   Copyright (c) 1998 - 2012
@@ -106,6 +106,8 @@ namespace folia {
     Sentence *sentences( size_t ) const;
     Sentence *rsentences( size_t ) const;
     std::string toXml( const std::string& ="" ) const;
+    MetaDataType metadatatype() const { return _metadatatype; };
+    std::string metadatafile() const { return _metadatafile; };
 
     FoliaElement *append( FoliaElement* );
 
@@ -115,13 +117,15 @@ namespace folia {
   
     FoliaElement *index( const std::string& ) const; //retrieve element with specified ID 
     FoliaElement* operator []( const std::string& ) const ; //index as operator 
-    bool isDeclared( const std::string&, AnnotationType::AnnotationType );
+    bool isDeclared( AnnotationType::AnnotationType, const std::string&, 
+		     const std::string&, const std::string& );
+    bool isDeclared( AnnotationType::AnnotationType, const std::string& = "" );
     std::string defaultset( AnnotationType::AnnotationType ) const;
 
     std::string defaultannotator( AnnotationType::AnnotationType, 
-				  const std::string& ="", bool = false ) const;
+				  const std::string& ="" ) const;
     std::string defaultannotatortype( AnnotationType::AnnotationType, 
-				      const std::string& ="", bool = false ) const;
+				      const std::string& ="" ) const;
 
     FoliaElement* parseXml( );
 
@@ -129,6 +133,8 @@ namespace folia {
     void declare( AnnotationType::AnnotationType, 
 		  const std::string&,
 		  const std::string& = "" );
+    void declare( AnnotationType::AnnotationType, 
+		  const std::string&, const std::string&, const std::string& );
     xmlDoc *XmlDoc() const { return xmldoc; };
     xmlNs *foliaNs() const { return _foliaNs; };
     void keepForDeletion( FoliaElement *p ) { delSet.insert( p ); };
@@ -141,16 +147,8 @@ namespace folia {
       std::string a;
       std::string t;
     };
-    class ts_t {
-      friend std::ostream& operator<<( std::ostream&, const ts_t& );
-    public:
-    ts_t( AnnotationType::AnnotationType _t, const std::string& _s): t(_t),s(_s){};
-      AnnotationType::AnnotationType t;
-      std::string s;
-    };
-
   private:
-    std::map<AnnotationType::AnnotationType,std::map<std::string,at_t> > annotationdefaults;
+    std::map<AnnotationType::AnnotationType,std::multimap<std::string,at_t> > annotationdefaults;
 
     FoliaElement* parseFoliaDoc( xmlNode * );
     void setimdi( xmlNode * );
@@ -164,13 +162,13 @@ namespace folia {
     std::vector<FoliaElement* > iindex;
     std::vector<FoliaElement*> data;
     std::string _id;
-    std::list<ts_t> annotations;
     std::set<FoliaElement *> delSet;
     FoliaElement *foliadoc;
     xmlDoc *xmldoc;
     mutable xmlNs *_foliaNs;
-    MetaDataType metadatatype;
+    MetaDataType _metadatatype;
     xmlNode *metadata;
+    std::string _metadatafile;
     std::string _title;
     std::string _date;
     std::string _language;
@@ -192,11 +190,6 @@ namespace folia {
     return os;
   }
 
-  inline std::ostream& operator<<( std::ostream& os, const Document::ts_t& ts ){
-    os << "<" << ts.t << "," << ts.s << ">";
-    return os;
-  }
-
 } // namespace folia
 
 #endif // DOCUMENT_H
