@@ -1,5 +1,5 @@
 /*
-  $Id: foliautils.h 13851 2012-01-03 08:48:59Z sloot $
+  $Id: foliautils.h 14458 2012-03-13 17:05:21Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/libfolia/trunk/include/folia/foliautils.h $
 
   Copyright (c) 1998 - 2012
@@ -94,10 +94,10 @@ namespace folia {
    */
   namespace AnnotationType {
     enum AnnotationType { NO_ANN, TEXT, TOKEN, DIVISION, POS, LEMMA, EVENT,
-			  SUGGESTION, 
-			  DOMEIN, SENSE, SYNTAX, CHUNKING, SUBENTITY,
-			  ENTITY, ERRORDETECTION, CORRECTION, ALTERNATIVE, PHON,
-			  SUBJECTIVITY, MORPHOLOGICAL, DEPENDENCY, TIMEDEVENT
+			  SUGGESTION, DOMEIN, SENSE, SYNTAX, CHUNKING, 
+			  SUBENTITY, ENTITY, ERRORDETECTION, CORRECTION,
+			  ALTERNATIVE, PHON, SUBJECTIVITY, MORPHOLOGICAL, 
+			  DEPENDENCY, TIMEDEVENT, GAP
     };
   }
   
@@ -190,6 +190,12 @@ namespace folia {
   KWargs getArgs( const std::string& );
   std::string toString( const KWargs& );
   
+  inline std::ostream& operator<<( std::ostream& os, 
+				   const AnnotationType::AnnotationType& at ){
+    os << toString( at );
+    return os;
+  }
+  
   template< typename T >
     inline T stringTo( const std::string& str ) {
     T result;
@@ -280,6 +286,20 @@ namespace folia {
     return os;
   }
 
+  template< typename S, typename T >
+    inline std::ostream& operator<< ( std::ostream& os, const std::multimap<S,T>& s ){
+    os << "{";
+    typename std::multimap<S,T>::const_iterator it = s.begin();
+    while ( it != s.end() ){
+      os << "<" << it->first << "," << it->second << ">";
+      ++it;
+      if ( it != s.end() )
+	os << ",";
+    }
+    os << "}";
+    return os;
+  }
+
   xmlNode *newXMLNode( xmlNs *,  const std::string& );
   void addAttributes( xmlNode *, const KWargs& );
   KWargs getAttributes( const xmlNode * );
