@@ -1,5 +1,5 @@
 /*
-  $Id: document.cxx 13851 2012-01-03 08:48:59Z sloot $
+  $Id: document.cxx 14474 2012-03-19 13:04:14Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/libfolia/trunk/src/document.cxx $
 
   Copyright (c) 1998 - 2012
@@ -57,11 +57,12 @@ namespace folia {
   Document::Document( const string& args ){
     init();
     setAttributes( getArgs( args ) );
-    foliadoc = new FoLiA( this, args );
+    if ( ! foliadoc )
+      foliadoc = new FoLiA( this, args );
   }
 
   void Document::init(){
-    metadatatype = NATIVE;
+    _metadatatype = NATIVE;
     metadata = 0;
     xmldoc = 0;
     foliadoc = 0;
@@ -97,7 +98,7 @@ namespace folia {
       return false;
     if ( a1._set != a2._set )
       return false;
-    if ( a1._cls != a2._cls )
+    if ( a1._class != a2._class )
       return false;
     if ( a1._annotator != a2._annotator )
       return false;
@@ -373,31 +374,25 @@ namespace folia {
       if ( tag.length() > 11 && tag.substr( tag.length() - 11 ) == "-annotation" ){
 	string prefix = tag.substr( 0,  tag.length() - 11 );
 	AnnotationType::AnnotationType type = stringToAT( prefix );
-	annotations.push_back( ts_t( type, "" ) );
-	annotationdefaults[type].clear();
 	KWargs att = getAttributes( n );
-	KWargs::const_iterator it = att.find("set" );
 	string s;
 	string a;
 	string t;
-	if ( it != att.end() )
+	KWargs::const_iterator it = att.find("set" );
+	if ( it != att.end() ){
 	  s = it->second;
-	else
-	  runtime_error( "what if no set? " );
+	}
+	else {
+	  s = "undefined"; // default value
+	}
 	it = att.find( "annotator" );
 	if ( it != att.end() )
 	  a = it->second;
 	it = att.find( "annotatortype" );
 	if ( it != att.end() ){
-	  if ( it->second == "auto" )
-	    t = "auto";
-	  else if ( it->second == "manual" )
-	    t = "manual";
-	  else
-	    t = it->second;
+	  t = it->second;
 	}
-	annotationdefaults[type].insert( make_pair(s,at_t( a, t )));
-	//      cerr << "inserted [" << toString(type) << "][" << s << "](" << a << "," << t << ")" << endl;
+	declare( type, s, a, t );
       }
       n = n->next;
     }
@@ -430,14 +425,28 @@ namespace folia {
 	     checkNS( p, NSFOLIA ) ){
 	  if ( debug > 1 )
 	    cerr << "Found metadata" << endl;
+	  KWargs atts = getAttributes( p );
+	  string val = lowercase(atts["type"]);
+	  if ( !val.empty() ){
+	    if ( val == "native" ){
+	      _metadatatype = NATIVE;
+	    }
+	    else if ( val == "imdi" ){
+	      _metadatatype = IMDI;
+	      _metadatafile = atts["src"];
+	    }
+	    else if ( val == "cmdi" ){
+	      _metadatatype = CMDI;
+	      _metadatafile = atts["src"];
+	    }
+	  }
 	  xmlNode *m = p->children;
 	  while ( m ){
 	    if ( Name(m)  == "METATRANSCRIPT" ){
-	      if ( !checkNS( m, NSIMDI ) )
+	      if ( !checkNS( m, NSIMDI ) || _metadatatype != IMDI )
 		throw runtime_error( "imdi != imdi " );
 	      if ( debug > 1 )
 		cerr << "found IMDI" << endl;
-	      metadatatype = IMDI;
 	      metadata = m;
 	      setimdi(m);
 	    }
@@ -503,7 +512,6 @@ namespace folia {
 	}
 	else if ( Name( root ) == "DCOI" &&
 		  checkNS( root, NSDCOI ) ){
-	  cerr << " D-coi is not directly supported";
 	  throw runtime_error( "DCOI format not supported" );
 	}
 	else {
@@ -518,20 +526,29 @@ namespace folia {
   }
 
   void Document::declare( AnnotationType::AnnotationType type, 
-			  const string& st, const string& args ){
-    bool found = false;
-    list<ts_t>::const_iterator it = annotations.begin();
-    while( it != annotations.end() && !found ){
-      found = ( it->t == type && it->s == st );
-      ++it;
-    }
-    if ( !found ){
-      annotations.push_back( ts_t(type,st) );
-    }
+			  const string& setname, const string& args ){
+    string st = setname;
+    if ( st.empty() )
+      st = "undefined";
     KWargs kw = getArgs( args );
     string a = kw["annotator"];
     string t = kw["annotatortype"];
-    annotationdefaults[type].insert( make_pair(st, at_t(a,t) ) );
+    kw.erase("annotator");
+    kw.erase("annotatortype");
+    if ( kw.size() != 0 ){
+      throw XmlError( "declaration: expected 'annotator' of 'annotatortype', got '" + kw.begin()->first + "'" );
+    }
+    declare( type, st, a, t );
+  }
+
+  void Document::declare( AnnotationType::AnnotationType type, 
+			  const string& s, const string& a, const string& t ){
+    if ( !isDeclared( type, s, a, t ) ){
+      annotationdefaults[type].insert( make_pair(s, at_t(a,t) ) );
+      //    cerr << "inserted [" << type << "][" << st << "](" << a << "," << t << ")" << endl;
+      //    cerr << "annotation defaults now: " <<  annotationdefaults << endl;
+      
+    }
   }
 
   FoliaElement* Document::addNode( ElementType et, const KWargs& kwargs ){
@@ -541,11 +558,31 @@ namespace folia {
     return res;
   }
 
-  bool Document::isDeclared( const string& s, 
-			     AnnotationType::AnnotationType type ){
-    map<AnnotationType::AnnotationType,map<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
+  bool Document::isDeclared( AnnotationType::AnnotationType type,
+			     const string& s, 
+			     const string& a,
+			     const string& t ){
+    map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
     if ( mit1 != annotationdefaults.end() ){
-      map<string,at_t>::const_iterator mit2 = mit1->second.find(s);
+      if ( s.empty() )
+	throw runtime_error("isDeclared with empty set.");
+      multimap<string,at_t>::const_iterator mit2 = mit1->second.lower_bound(s);
+      while ( mit2 != mit1->second.upper_bound(s) ){
+	if ( mit2->second.a == a && mit2->second.t == t )
+	  return true;
+	++mit2;
+      }
+    }
+    return false;
+  }
+
+  bool Document::isDeclared( AnnotationType::AnnotationType type,
+			     const string& s ){
+    map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
+    if ( mit1 != annotationdefaults.end() ){
+      if ( s.empty() )
+	return true;
+      multimap<string,at_t>::const_iterator mit2 = mit1->second.find(s);
       return mit2 != mit1->second.end();
     }
     return false;
@@ -553,90 +590,101 @@ namespace folia {
 
   string Document::defaultset( AnnotationType::AnnotationType type ) const {
     // search a set. it must be unique. Otherwise return ""
-    map<AnnotationType::AnnotationType,map<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
+    //    cerr << "zoek '" << type << "' default set " <<  annotationdefaults << endl;
+    map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
     string result;
     if ( mit1 != annotationdefaults.end() ){
+      //      cerr << "vind tussen " <<  mit1->second << endl;
       if ( mit1->second.size() == 1 )
 	result = mit1->second.begin()->first;
     }
-    //  cerr << "defaultset ==> " << result << endl;
+    //    cerr << "defaultset ==> " << result << endl;
     return result;
   }
 
   std::string Document::defaultannotator( AnnotationType::AnnotationType type,
-					  const string& st,
-					  bool noThrow ) const {
-    map<AnnotationType::AnnotationType,map<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
+					  const string& st ) const {
+    // if ( !st.empty() ){
+    //   cerr << "zoek '" << st << "' default annotator " <<  annotationdefaults << endl;
+    // }
+    map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
     string result;
     if ( mit1 != annotationdefaults.end() ){
-      if ( st.empty() )
-	result = mit1->second.begin()->second.a;
+      //      cerr << "vind tussen " <<  mit1->second << endl;
+      if ( st.empty() ){
+	if ( mit1->second.size() == 1 ){
+	  result = mit1->second.begin()->second.a;
+	  return result;
+	}
+      }
       else {
-	map<string,at_t>::const_iterator mit2 = mit1->second.find( st );
-	if ( mit2 != mit1->second.end() ){
+	if ( mit1->second.count( st ) == 1 ){
+	  map<string,at_t>::const_iterator mit2 = mit1->second.find( st );
 	  result = mit2->second.a;
 	}
       }
     }
-    //  cerr << "get default ==> " << result << endl;
-    if ( !noThrow && result.empty() )
-      throw NoDefaultError( "defaultannotator" );
+    //    cerr << "get default ==> " << result << endl;
     return result;
   }
 
   std::string Document::defaultannotatortype( AnnotationType::AnnotationType type,
-					      const string& st,
-					      bool noThrow ) const {
-    map<AnnotationType::AnnotationType,map<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
+					      const string& st ) const {
+    map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit1 = annotationdefaults.find(type);
     string result;
     if ( mit1 != annotationdefaults.end() ){
-      if ( st.empty() )
-	result = mit1->second.begin()->second.t;
+      if ( st.empty() ){
+	if ( mit1->second.size() == 1 )
+	  result = mit1->second.begin()->second.t;
+	return result;
+      }
       else {
-	map<string,at_t>::const_iterator mit2 = mit1->second.find( st );
-	if ( mit2 != mit1->second.end() ){
+	if ( mit1->second.count( st ) == 1 ){
+	  map<string,at_t>::const_iterator mit2 = mit1->second.find( st );
 	  result = mit2->second.t;
 	}
       }
     }
     //  cerr << "get default ==> " << result << endl;
-    if ( !noThrow && result.empty() )
-      throw NoDefaultError( "defaultannotationtype" );
     return result;
   }
 
-  void Document::setannotations( xmlNode *node ) const{
-    list<ts_t>::const_iterator it = annotations.begin();
-    while ( it != annotations.end() ){
-      // Find the 'label' 
-      string label = toString( it->t );
-      label += "-annotation";
-      xmlNode *n = xmlAddChild( node, newXMLNode( _foliaNs, label ) );
-      KWargs args;
-      string s = defaultannotator( it->t, "", true );
-      if ( !s.empty() )
-	args["annotator"] = s;
-      s = defaultannotatortype( it->t, "", true );
-      if ( !s.empty() )
-	args["annotatortype"] = s;
-      s = defaultset( it->t );
-      if ( !s.empty() )
-	args["set"] = s;
-      addAttributes( n, args );
-      ++it;
-    } 
+  void Document::setannotations( xmlNode *node ) const {
+    map<AnnotationType::AnnotationType,multimap<string,at_t> >::const_iterator mit = annotationdefaults.begin();
+    while ( mit != annotationdefaults.end() ){
+      map<string,at_t>::const_iterator it = mit->second.begin();
+      while ( it != mit->second.end() ){
+	// Find the 'label' 
+	string label = toString( mit->first );
+	label += "-annotation";
+	xmlNode *n = xmlAddChild( node, newXMLNode( _foliaNs, label ) );
+	KWargs args;
+	string s = it->second.a;
+	if ( !s.empty() )
+	  args["annotator"] = s;
+	s = it->second.t;
+	if ( !s.empty() )
+	  args["annotatortype"] = s;
+	s = it->first;
+	if ( s != "undefined" ) // the default
+	  args["set"] = s;
+	addAttributes( n, args );
+	++it;
+      }
+      ++mit;
+    }
   }
 
   void Document::setmetadata( xmlNode *node ) const{
     KWargs atts;
-    if ( metadatatype == NATIVE )
+    if ( _metadatatype == NATIVE )
       atts["type"] = "native";
-    else if ( metadatatype == IMDI )
+    else if ( _metadatatype == IMDI )
       atts["type"] = "imdi";
-    else if ( metadatatype == CMDI )
+    else if ( _metadatatype == CMDI )
       atts["type"] = "cmdi";
 
-    if ( metadatatype == NATIVE ){
+    if ( _metadatatype == NATIVE ){
       if ( !_title.empty() )
 	atts["title"] = _title;
       if ( !_date.empty() )
@@ -648,11 +696,11 @@ namespace folia {
       if ( !_publisher.empty() )
 	atts["publisher"] = _publisher;
     }
-    else if ( metadatatype == IMDI ){
+    else if ( _metadatatype == IMDI  ||
+	      _metadatatype == CMDI ){
       xmlAddChild( node, xmlCopyNodeList(metadata) );
-    }
-    else if ( metadatatype == CMDI ){
-      // ????
+      if ( !_metadatafile.empty() )
+	atts["src"] = _metadatafile;
     }
     addAttributes( node, atts );
   }
