@@ -1,5 +1,5 @@
 /*
-  $Id: folia.cxx 13851 2012-01-03 08:48:59Z sloot $
+  $Id: folia.cxx 14476 2012-03-19 15:48:35Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/libfolia/trunk/src/folia.cxx $
 
   Copyright (c) 1998 - 2012
@@ -117,7 +117,7 @@ namespace folia {
 
   void FoliaElement::setAttributes( const KWargs& kwargs ){
     Attrib supported = _required_attributes | _optional_attributes;
-    // if ( _element_id == Quote_t ){
+    // if ( _element_id == Gap_t ){
     //   cerr << "set attributes: " << kwargs << " on " << toString(_element_id) << endl;
     //   cerr << "required = " <<  _required_attributes << endl;
     //   cerr << "optional = " <<  _optional_attributes << endl;
@@ -163,8 +163,10 @@ namespace folia {
       else {
 	_set = it->second;
       }
-      if ( mydoc &&
-	   !mydoc->isDeclared( _set , _annotation_type ) )
+      if ( !mydoc ){
+	throw ValueError( "Set=" + _set + " is used on a node without a document." );
+      }
+      if ( !mydoc->isDeclared( _annotation_type, _set ) )
 	throw ValueError( "Set " + _set + " is used but has no declaration " +
 			  "for " + toString( _annotation_type ) + "-annotation" );
     }
@@ -175,23 +177,36 @@ namespace folia {
       throw ValueError("Set is required for " + classname() );
     else
       _set = "";
-
+    
     it = kwargs.find( "class" );
     if ( it == kwargs.end() )
       it = kwargs.find( "cls" );
     if ( it != kwargs.end() ) {
       if ( !( CLASS & supported ) )
 	throw ValueError("Class is not supported for " + classname() );
-      else {
-	_cls = it->second;
+      _class = it->second;
+      if ( _element_id != TextContent_t ){
+	if ( !mydoc ){
+	  throw ValueError( "Class=" + _class + " is used on a node without a document." );
+	}
+	else if ( _set == "" && 
+		  mydoc->defaultset( _annotation_type ) == "" && 
+		  mydoc->isDeclared( _annotation_type ) ){
+	  throw ValueError( "Class " + _class + " is used but has no default declaration " +
+			    "for " + toString( _annotation_type ) + "-annotation" );
+	}
       }
     }
     else if ( CLASS & _required_attributes )
       throw ValueError("Class is required for " + classname() );
     else
-      _cls = "";
+      _class = "";
 
-      
+    if ( _element_id != TextContent_t ){
+      if ( !_class.empty() && _set.empty() )
+	throw ValueError("Set is required for " + classname() + 
+			 " class=\"" + _class + "\" assigned without set."  );
+    }
     it = kwargs.find( "annotator" );    
     if ( it != kwargs.end() ) {
       if ( !(ANNOTATOR & supported) )
@@ -201,7 +216,7 @@ namespace folia {
       }
     }
     else if ( mydoc &&
-	      (def = mydoc->defaultannotator( _annotation_type, "", true )) != "" ){
+	      (def = mydoc->defaultannotator( _annotation_type, _set )) != "" ){
       _annotator = def;
     }
     else if ( ANNOTATOR & _required_attributes )
@@ -220,7 +235,7 @@ namespace folia {
       }
     }
     else if ( mydoc &&
-	      (def = mydoc->defaultannotatortype( _annotation_type, "", true ) ) != ""  ){
+	      (def = mydoc->defaultannotatortype( _annotation_type, _set ) ) != ""  ){
       _annotator_type = stringTo<AnnotatorType>( def );
       if ( _annotator_type == UNDEFINED )
 	throw ValueError("annotatortype must be 'auto' or 'manual'");
@@ -236,9 +251,14 @@ namespace folia {
       if ( !(CONFIDENCE & supported) )
 	throw ValueError("Confidence is not supported for " + classname() );
       else {
-	_confidence = toDouble(it->second);
-	if ( ( _confidence < 0.0 || _confidence > 1.0 ) )
-	  throw ValueError("Confidence must be a floating point number between 0 and 1");
+	try {
+	  _confidence = stringTo<double>(it->second);
+	  if ( _confidence < 0 || _confidence > 1.0 )
+	    throw ValueError("Confidence must be a floating point number between 0 and 1");	    
+	}
+	catch (...){
+	  throw ValueError("invalid Confidence value, (not a number?)");
+	}
       }
     }
     else if ( CONFIDENCE & _required_attributes )
@@ -278,33 +298,48 @@ namespace folia {
     if ( it != kwargs.end() ){
       _auth = stringTo<bool>( it->second );
     }
-
+    
     if ( mydoc && !_id.empty() )
       mydoc->addDocIndex( this, _id );
-
+    
     it = kwargs.find( "actor" );
     if ( it != kwargs.end() ){
-      FoliaElement *tmp = new ActorFeature( "cls='" + it->second + "'" );
+      KWargs newa;
+      newa["class"] = it->second;
+      FoliaElement *tmp = new ActorFeature();
+      tmp->setAttributes( newa );
       append( tmp );
     }
     it = kwargs.find( "head" );
     if ( it != kwargs.end() ){
-      FoliaElement *tmp = new HeadFeature( "cls='" + it->second + "'" );
+      KWargs newa;
+      newa["class"] = it->second;
+      FoliaElement *tmp = new HeadFeature();
+      tmp->setAttributes( newa );
       append( tmp );
     }
     it = kwargs.find( "synset" );
     if ( it != kwargs.end() ){
-      FoliaElement *tmp = new SynsetFeature( "cls='" + it->second + "'" );
+      KWargs newa;
+      newa["class"] = it->second;
+      FoliaElement *tmp = new SynsetFeature();
+      tmp->setAttributes( newa );
       append( tmp );
     }
     it = kwargs.find( "begindatetime" );
     if ( it != kwargs.end() ){
-      FoliaElement *tmp = new BegindatetimeFeature( "cls='" + it->second + "'" );
+      KWargs newa;
+      newa["class"] = it->second;
+      FoliaElement *tmp = new BegindatetimeFeature();
+      tmp->setAttributes( newa );
       append( tmp );
     }
     it = kwargs.find( "enddatetime" );
     if ( it != kwargs.end() ){
-      FoliaElement *tmp = new EnddatetimeFeature( "cls='" + it->second + "'" );
+      KWargs newa;
+      newa["class"] = it->second;
+      FoliaElement *tmp = new EnddatetimeFeature();
+      tmp->setAttributes( newa );
       append( tmp );
     }
   
@@ -314,6 +349,7 @@ namespace folia {
     KWargs attribs;
     bool isDefaultSet = true;
     bool isDefaultAnn = true;
+
     if ( !_id.empty() ){
       attribs["_id"] = _id; // sort "id" as first!
     }
@@ -322,17 +358,17 @@ namespace folia {
       isDefaultSet = false;
       attribs["set"] = _set;
     }
-    if ( !_cls.empty() )
-      attribs["class"] = _cls;
+    if ( !_class.empty() )
+      attribs["class"] = _class;
 
     if ( !_annotator.empty() &&
-	 _annotator != mydoc->defaultannotator( _annotation_type, _set, true ) ){
+	 _annotator != mydoc->defaultannotator( _annotation_type, _set ) ){
       isDefaultAnn = false;
       attribs["annotator"] = _annotator;
     }
   
     if ( _annotator_type != UNDEFINED ){
-      AnnotatorType at = stringTo<AnnotatorType>( mydoc->defaultannotatortype( _annotation_type, _set, true ) );
+      AnnotatorType at = stringTo<AnnotatorType>( mydoc->defaultannotatortype( _annotation_type, _set ) );
       if ( (!isDefaultSet || !isDefaultAnn) && _annotator_type != at ){
 	if ( _annotator_type == AUTO )
 	  attribs["annotatortype"] = "auto";
@@ -407,7 +443,7 @@ namespace folia {
       while ( it != data.end() ){
 	if ( skipelements.find(*it) == skipelements.end() ){
 	  if ( (*it)->isinstance(TextContent_t) ){
-	    if ( (*it)->_cls == "current" )
+	    if ( (*it)->_class == "current" )
 	      textelements.push_front( *it );
 	    else
 	      textelements.push_back( *it );
@@ -439,48 +475,46 @@ namespace folia {
       return false;
     }
   }
-
+  
   UnicodeString FoliaElement::text( const string& cls ) const {
     if ( !PRINTABLE )
       throw NoSuchText( _xmltag );
     //  cerr << (void*)this << ":text() for " << _xmltag << " and class= " << cls << " step 1 " << endl;
-
+    
     UnicodeString result;    
     try {
-       result = this->textcontent(cls)->text(cls);
+      result = this->textcontent(cls)->text(cls);
     } catch (NoSuchText& e ) {
-	for( size_t i=0; i < data.size(); ++i ){
-	  // try to get text dynamically from children
-	  // skip TextContent elements
-	  if ( data[i]->PRINTABLE && !data[i]->isinstance( TextContent_t ) ){
-	    try {
-	      UnicodeString tmp = data[i]->text( cls );
-	      //	cerr << "text() for " << _xmltag << " step 4, tmp= " << tmp << endl;
-	      result += tmp;
-	      if ( !tmp.isEmpty() ){
-		result += UTF8ToUnicode( data[i]->getTextDelimiter() );
-	      }
-	    } catch ( NoSuchText& e ){
+      for( size_t i=0; i < data.size(); ++i ){
+	// try to get text dynamically from children
+	// skip TextContent elements
+	if ( data[i]->PRINTABLE && !data[i]->isinstance( TextContent_t ) ){
+	  try {
+	    UnicodeString tmp = data[i]->text( cls );
+	    //	cerr << "text() for " << _xmltag << " step 2, tmp= " << tmp << endl;
+	    result += tmp;
+	    if ( !tmp.isEmpty() ){
+	      result += UTF8ToUnicode( data[i]->getTextDelimiter() );
 	    }
+	  } catch ( NoSuchText& e ){
 	  }
 	}
+      }
     }
     
-    //  cerr << "text() for " << _xmltag << " step 2 >> " << endl;
-
-    // try to get text from children. 
-    //  cerr << "text() for " << _xmltag << " step 5, result= " << result << endl;
+    //  cerr << "text() for " << _xmltag << " step 3 >> " << endl;
+    
     result.trim();
     if ( !result.isEmpty() ){
-      //    cerr << "text() for " << _xmltag << " step 6, result= " << result << endl;
+      //    cerr << "text() for " << _xmltag << " result= " << result << endl;
       return result;
     }
     else
-      throw NoSuchText( ":{" );
+      throw NoSuchText( ": empty!" );
   }
-
+  
   UnicodeString FoliaElement::stricttext( const string& cls ) const {
-      return this->textcontent(cls)->text(cls);
+    return this->textcontent(cls)->text(cls);
   }
 
   TextContent *FoliaElement::textcontent( const string& cls ) const {
@@ -493,7 +527,7 @@ namespace folia {
       throw NoSuchText( _xmltag );
     
     for( size_t i=0; i < data.size(); ++i ){
-      if ( data[i]->isinstance(TextContent_t) && (data[i]->_cls == cls) ){
+      if ( data[i]->isinstance(TextContent_t) && (data[i]->_class == cls) ){
 	return dynamic_cast<TextContent*>(data[i]);
       }
       else if ( data[i]->element_id() == Correction_t) {
@@ -531,10 +565,10 @@ namespace folia {
   }                
 
   TextContent *FoliaElement::settext( const string& txt, 
-					 const string& cls ){
+				      const string& cls ){
     KWargs args;
     args["value"] = txt;
-    args["cls"] = cls;
+    args["class"] = cls;
     TextContent *node = new TextContent( mydoc );
     node->setAttributes( args );
     replace( node );
@@ -587,7 +621,7 @@ namespace folia {
       string myid = id();
       if ( !_set.empty() 
 	   && (CLASS & _required_attributes )
-	   && !mydoc->isDeclared( _set , _annotation_type ) )
+	   && !mydoc->isDeclared( _annotation_type, _set ) )
 	throw ValueError( "Set " + _set + " is used in " + _xmltag 
 			  + "element: " + myid + " but has no declaration " +
 			  "for " + toString( _annotation_type ) + "-annotation" );
@@ -660,8 +694,8 @@ namespace folia {
   }
 
   vector<FoliaElement*> FoliaElement::select( ElementType et,
-						    const string& val,
-						    bool recurse ) const {
+					      const string& val,
+					      bool recurse ) const {
     static set<ElementType> excludeSet;
     if ( excludeSet.empty() ){
       excludeSet.insert( Original_t );
@@ -672,8 +706,8 @@ namespace folia {
   }
 
   vector<FoliaElement*> FoliaElement::select( ElementType et,
-						    const set<ElementType>& exclude,
-						    bool recurse ) const {
+					      const set<ElementType>& exclude,
+					      bool recurse ) const {
     vector<FoliaElement*> res;
     for ( size_t i = 0; i < data.size(); ++i ){
       if ( data[i]->_element_id == et ){
@@ -690,7 +724,7 @@ namespace folia {
   }
 
   vector<FoliaElement*> FoliaElement::select( ElementType et,
-						    bool recurse ) const {
+					      bool recurse ) const {
     static set<ElementType> excludeSet;
     if ( excludeSet.empty() ){
       excludeSet.insert( Quote_t );
@@ -747,23 +781,6 @@ namespace folia {
     return buf;
   }
 
-  Alternative *FoliaElement::addAlternative( ElementType et,
-					     const KWargs& args ){
-    Alternative *res = new Alternative( mydoc );
-    KWargs kw;
-    string id = generateId( "alt" );
-    kw["id"] = id;
-    res->setAttributes( kw );
-    if ( et == Pos_t )
-      res->addPosAnnotation( args );
-    else if ( et == Lemma_t )
-      res->addLemmaAnnotation( args );
-    else
-      throw runtime_error( "addAlternative not implemenentd for " + toString(et) );
-    append( res );
-    return res;
-  }
-
   string FoliaElement::pos() const { 
     return annotation<PosAnnotation>()->cls(); 
   }
@@ -772,40 +789,12 @@ namespace folia {
     return annotation<LemmaAnnotation>()->cls(); 
   }
 
-  AbstractTokenAnnotation *FoliaElement::addAnnotation( ElementType et,
-							   const KWargs& args ){
-    if ( et == Pos_t )
-      return addPosAnnotation( args );
-    else if ( et == Lemma_t )
-      return addLemmaAnnotation( args );
-    else
-      throw runtime_error( "addAnnotation not implemenentd for " + toString(et) );
+  PosAnnotation *FoliaElement::addPosAnnotation( const KWargs& args ){
+    return addAnnotation<PosAnnotation>( args );
   }
 
-  AbstractTokenAnnotation *FoliaElement::addPosAnnotation( const KWargs& args ){
-    PosAnnotation *res = new PosAnnotation( mydoc );
-    try {
-      res->setAttributes( args );
-    }
-    catch( exception& ){
-      delete res;
-      throw;
-    }
-    append( res );
-    return res;
-  }
-
-  AbstractTokenAnnotation *FoliaElement::addLemmaAnnotation( const KWargs& args ){
-    LemmaAnnotation *res = new LemmaAnnotation( mydoc );
-    try {
-      res->setAttributes( args );
-    }
-    catch( exception& ){
-      delete res;
-      throw;
-    }
-    append( res );
-    return res;
+  LemmaAnnotation *FoliaElement::addLemmaAnnotation( const KWargs& args ){
+    return addAnnotation<LemmaAnnotation>( args );
   }
 
   Sentence *FoliaElement::addSentence( const KWargs& args ){
@@ -963,7 +952,7 @@ namespace folia {
     if ( it == kwargs.end() ) 
       it = kwargs.find( "class" );
     if ( it == kwargs.end() ) {
-      kwargs["cls"] = "current";
+      kwargs["class"] = "current";
     }
     FoliaElement::setAttributes(kwargs);
   }
@@ -977,12 +966,12 @@ namespace folia {
     return this;
   }
 
-  FoliaElement *TextContent::postappend(){
+  TextContent *TextContent::postappend(){
     if ( _parent->isinstance( Original_t ) ){
-      if ( _cls == "current" )
-	_cls = "original";
+      if ( _class == "current" )
+	_class = "original";
     }
-    return FoliaElement::postappend();
+    return this;
   }
 
   vector<FoliaElement *>TextContent::findreplacables( FoliaElement *par ) const {
@@ -992,7 +981,7 @@ namespace folia {
     vector<TextContent*>::iterator it = v.begin();
     while ( it != v.end() ){
       // cerr << "TextContent::findreplacable bekijkt " << *it << " (" 
-      if ( (*it)->cls() == _cls )
+      if ( (*it)->cls() == _class )
 	result.push_back( *it );
       ++it;
     }
@@ -1391,25 +1380,8 @@ namespace folia {
     return result;
   }
 
-  // FoliaElement *AbstractStructureElement::annotation( ElementType et ) const {
-  //   vector<FoliaElement *>v = annotations( et );
-  //   return v[0]; // always exist, otherwise annotations would throw()
-  // }
-
-  // FoliaElement *AbstractStructureElement::annotation( ElementType et,
-  // 						      const string& val ) const {
-  //   // Will return a SINGLE annotation (even if there are multiple). 
-  //   // Raises a NoSuchAnnotation exception if none was found
-  //   vector<FoliaElement *>v = select( et, val );
-  //   if ( v.size() >= 1 )
-  //     return v[0];
-  //   else
-  //     throw NoSuchAnnotation( toString(et) );
-  //   return 0;
-  // }
-
   vector<Alternative *> AbstractStructureElement::alternatives( ElementType elt,
-								 const string& st ) const {
+								const string& st ) const {
     // Return a list of alternatives, either all or only of a specific type, restrained by set
     static set<ElementType> excludeSet;
     if ( excludeSet.empty() ){
@@ -1426,7 +1398,7 @@ namespace folia {
 	if ( alts[i]->size() > 0 ) { // child elements?
 	  for ( size_t j =0; j < alts[i]->size(); ++j ){
 	    if ( alts[i]->index(j)->element_id() == elt &&
-		 ( alts[i]->st().empty() || alts[i]->st() == st ) ){
+		 ( alts[i]->sett().empty() || alts[i]->sett() == st ) ){
 	      res.push_back( alts[i] ); // not the child!
 	      break; // yield an alternative only once (in case there are multiple matches)
 	    }
@@ -1491,7 +1463,7 @@ namespace folia {
     if ( child->element_id() == Pos_t ||
 	 child->element_id() == Lemma_t ){
       // sanity check, there may be no other child within the same set
-      vector<FoliaElement*> v = select( child->element_id(), child->st() );
+      vector<FoliaElement*> v = select( child->element_id(), child->sett() );
       if ( v.empty() ) {
 	// OK!
 	return FoliaElement::append( child );
@@ -1525,7 +1497,7 @@ namespace folia {
   }
 
   Division *Word::division() const {
-    // return the sentence this word is a part of, otherwise return null
+    // return the <div> this word is a part of, otherwise return null
     FoliaElement *p = _parent; 
     while( p ){
       if ( p->isinstance( Division_t ) )
@@ -1593,7 +1565,7 @@ namespace folia {
 	    if ( val.empty() )
 	      result.push_back( 0 );
 	    else {
-	      PlaceHolder *p = new PlaceHolder( val );
+	      PlaceHolder *p = new PlaceHolder( "text='" + val + "'" );
 	      mydoc->keepForDeletion( p );
 	      result.push_back( p );
 	    }
@@ -1606,7 +1578,7 @@ namespace folia {
 	      if ( val.empty() )
 		result.push_back( 0 );
 	      else {
-		PlaceHolder *p = new PlaceHolder( val );
+		PlaceHolder *p = new PlaceHolder( "text='" + val + "'" );
 		mydoc->keepForDeletion( p );
 		result.push_back( p );
 	      }
@@ -1636,7 +1608,7 @@ namespace folia {
 	    if ( val.empty() )
 	      result.push_back( 0 );
 	    else {
-	      PlaceHolder *p = new PlaceHolder( val );
+	      PlaceHolder *p = new PlaceHolder( "text='" + val + "'" );
 	      mydoc->keepForDeletion( p );
 	      result.push_back( p );
 	    }
@@ -1668,7 +1640,7 @@ namespace folia {
 	      if ( val.empty() )
 		result.push_back( 0 );
 	      else {
-		PlaceHolder *p = new PlaceHolder( val );
+		PlaceHolder *p = new PlaceHolder( "text='" + val + "'" );
 		mydoc->keepForDeletion( p );
 		result.push_back( p );
 	      }
@@ -1709,17 +1681,22 @@ namespace folia {
     return res;
   }
 
-  void PlaceHolder::setAttributes( const string&s ){
-    KWargs args;
-    args["text"] = s;
+  void PlaceHolder::setAttributes( const KWargs& args ){
+    KWargs::const_iterator it = args.find( "text" );
+    if ( it == args.end() ){
+      throw ValueError("text attribute is required for " + classname() );
+    }
+    else if ( args.size() != 1 ){
+      throw ValueError("only the text attribute is supported for " + classname() );
+    }
     Word::setAttributes( args );
   }
 
   KWargs TextContent::collectAttributes() const {
     KWargs attribs = FoliaElement::collectAttributes();
-    if ( _cls == "current" )
+    if ( _class == "current" )
       attribs.erase( "class" );
-    else if ( _cls == "original" && parent()->isinstance( Original_t ) )
+    else if ( _class == "original" && parent()->isinstance( Original_t ) )
       attribs.erase( "class" );    
       
     if ( _offset >= 0 ){
@@ -2051,9 +2028,10 @@ namespace folia {
   void Gap::init(){
     _xmltag = "gap";
     _element_id = Gap_t;
+    _annotation_type = AnnotationType::GAP;
     const ElementType accept[] = { Content_t, Description_t };
     _accepted_data = std::set<ElementType>(accept, accept+2); 
-    _optional_attributes = ID|CLASS|ANNOTATOR|CONFIDENCE|N;
+    _optional_attributes = CLASS|ID|ANNOTATOR|CONFIDENCE|N;
   }
 
   void Content::init(){
@@ -2071,7 +2049,7 @@ namespace folia {
 				   Quote_t, Event_t,
 				   Correction_t,
 				   Description_t };
-    _accepted_data = std::set<ElementType>(accept, accept+11); 
+    _accepted_data = std::set<ElementType>(accept, accept+13); 
     _required_attributes = ID;
     _optional_attributes = N;
   }
@@ -2079,6 +2057,7 @@ namespace folia {
   void Division::init(){
     _xmltag="div";
     _element_id = Division_t;
+    _optional_attributes = CLASS;
     const ElementType accept[] = { Division_t, Gap_t, Head_t, Paragraph_t,
 				   Sentence_t, List_t, Figure_t, Event_t,
 				   Description_t, LineBreak_t, TextContent_t,
@@ -2102,9 +2081,9 @@ namespace folia {
     _element_id = Event_t;
     const ElementType accept[] = { Gap_t, Division_t, Paragraph_t, Sentence_t, 
 				   List_t, Figure_t, Description_t, 
-				   ActorFeature_t, TextContent_t,
+				   ActorFeature_t, TextContent_t, Feature_t,
 				   BegindatetimeFeature_t, EnddatetimeFeature_t };
-    _accepted_data = std::set<ElementType>(accept, accept+11); 
+    _accepted_data = std::set<ElementType>(accept, accept+12); 
     _required_attributes = CLASS;
     _optional_attributes = ID|ANNOTATOR|N|DATETIME;
     _annotation_type = AnnotationType::EVENT;
@@ -2310,7 +2289,7 @@ namespace folia {
     if ( it == kwargs.end() ) {
       throw ValueError("class attribute is required for " + classname() );
     }
-    _cls = it->second;
+    _class = it->second;
   }
 
   KWargs Feature::collectAttributes() const {
