@@ -1,5 +1,5 @@
 /*
-  $Id: foliautils.cxx 13911 2012-01-09 11:33:39Z sloot $
+  $Id: foliautils.cxx 14316 2012-02-23 10:04:11Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/libfolia/trunk/src/foliautils.cxx $
 
   Copyright (c) 1998 - 2012
@@ -59,12 +59,6 @@ namespace folia {
     return result;
   }
 
-  double toDouble( const string& s ){
-    char *end;
-    double res = strtod( s.c_str(), &end );
-    return res;
-  }
-
   string toString( const double d ){
     stringstream dummy;
     if ( !( dummy << d ) ) {
@@ -79,11 +73,17 @@ namespace folia {
     case AnnotationType::NO_ANN:
       result = "NoNe";
       break; 
+    case AnnotationType::TEXT:
+      result = "text";
+      break; 
     case AnnotationType::TOKEN:
       result = "token";
       break; 
+    case AnnotationType::GAP:
+      result = "gap";
+      break; 
     case AnnotationType::DIVISION:
-      result = "division";
+      result = "div";
       break; 
     case AnnotationType::POS: 
       result = "pos";
@@ -137,7 +137,7 @@ namespace folia {
       result = "timedevent";
       break;
     default:
-      throw ValueError( " unknown translation for annotation" + 
+      throw ValueError( " unknown translation for annotation: " + 
 			folia::toString(int(at)) );
     };
     return result;
@@ -159,10 +159,14 @@ namespace folia {
   }
 
   AnnotationType::AnnotationType stringToAT( const string& at ){
+    if ( at == "text" )
+      return AnnotationType::TEXT;
     if ( at == "token" )
       return AnnotationType::TOKEN;
-    if ( at == "division" )
+    if ( at == "div" )
       return AnnotationType::DIVISION;
+    if ( at == "gap" )
+      return AnnotationType::GAP;
     if ( at == "pos" )
       return AnnotationType::POS;
     if ( at == "lemma" )
